% trigradient vectorized
function [zx,zy] = trigradient(x,y,z,t,opt)
% find the delaunay triangulation of the scaterred data if none provided
if nargin<4; t = delaunay(x,y); opt ='normal';end
p = [x,y];

% catch for no option given.
if nargin<5; opt='normal'; end

% number of triangles and points
nt = size(t,1); np = size(p,1);

% determine the coefficients for the plane above each triangle
% [x1 y1 1; x2 y2 1; x3 y3 1][a; b; c] = [z1; z2; z3] 
C = sparse(repmat(1:3*nt,3,1)',kron(reshape(1:3*nt,3,nt)',[1; 1; 1]),[p(t',:),ones(3*nt,1)])\z(t',:);

% return the derivatives of the triangles if the 'face' opt is selected
if strcmp(opt,'face')
    zx = C(1:3:end,:);
    zy = C(2:3:end,:);
    return
end

% calculate triangle areas using the cross product
areas = repmat(.5*abs((p(t(:,2),1)-p(t(:,1),1)).*(p(t(:,3),2)-p(t(:,1),2))-(p(t(:,3),1)-p(t(:,1),1)).*(p(t(:,2),2)-p(t(:,1),2))),1,size(z,2));

% map matrix from triangles to nodes
M = sparse(repmat((1:nt)',3,1),t,1,nt,np);

% weight the partial derivative (a or b coefficient) by the area of each
% triangle and sum for a given node. divide by the total area 
zx = (M'*(C(1:3:end,:).*areas))./(M'*areas);
zy = (M'*(C(2:3:end,:).*areas))./(M'*areas);

